#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_projectsearchviadatabase
----------------------------------

Tests for `ProjectSearchViaDatabase` class.
"""
import os
import shutil
import tempfile
import sys
import re
import unittest
from mock import Mock
import configparser
from pg8000 import InterfaceError

from ncmirtools.lookup import ProjectSearchViaDatabase
from ncmirtools.config import NcmirToolsConfig



class TestProjectSearchViaDatabase(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_get_connection(self):
        config = configparser.ConfigParser()
        config.add_section(NcmirToolsConfig.POSTGRES_SECTION)
        config.set(NcmirToolsConfig.POSTGRES_SECTION,
                   NcmirToolsConfig.POSTGRES_USER, 'yo')
        config.set(NcmirToolsConfig.POSTGRES_SECTION,
                   NcmirToolsConfig.POSTGRES_PASS, 'yo')
        config.set(NcmirToolsConfig.POSTGRES_SECTION,
                   NcmirToolsConfig.POSTGRES_HOST, 'localhost')
        config.set(NcmirToolsConfig.POSTGRES_SECTION,
                   NcmirToolsConfig.POSTGRES_PORT, '5')
        config.set(NcmirToolsConfig.POSTGRES_SECTION,
                   NcmirToolsConfig.POSTGRES_DB, 'yo')

        ps = ProjectSearchViaDatabase(config)
        mockcon = Mock()
        ps.set_alternate_connection(mockcon)
        c = ps._get_connection()
        self.assertEqual(c, mockcon)
        ps.set_config(config)
        ps.set_alternate_connection(None)
        try:
            c = ps._get_connection()
            self.fail('Expected InterfaceError, course this requires'
                      'postgres NOT to be running on localhost listening'
                      'on port 5')
        except InterfaceError as e:
            self.assertTrue(str(e).startswith("('communication error'"))

    def test_get_matching_projects_no_keyword(self):
        ps = ProjectSearchViaDatabase(configparser.ConfigParser())

        mockcon = Mock()
        mcursor = Mock()
        mcursor.fetchall = Mock(return_value=[(1, 'koo'), (3, 'yo')])
        mockcon.cursor = Mock(return_value=mcursor)

        ps.set_alternate_connection(mockcon)
        res = ps.get_matching_projects(None)

        mcursor.execute.assert_called_with("SELECT Project_id,project_name FROM Project")
        mcursor.close.assert_called_once_with()
        mockcon.commit.assert_called_once_with()
        mockcon.close.assert_called_once_with()
        self.assertEqual(res, ['1    koo', '3    yo'])

    def test_get_matching_projects_with_keyword(self):
        ps = ProjectSearchViaDatabase(configparser.ConfigParser())

        mockcon = Mock()
        mcursor = Mock()
        mcursor.fetchall = Mock(return_value=[(1, 'koo'), (3, 'yo'), ('4', 'val val')])
        mockcon.cursor = Mock(return_value=mcursor)

        ps.set_alternate_connection(mockcon)
        res = ps.get_matching_projects('ha ha')

        mcursor.execute.assert_called_with("SELECT Project_id,project_name FROM Project "
                               "WHERE project_name ILIKE '%%ha ha%%' OR "
                               " project_desc ILIKE '%%ha ha%%'")
        mcursor.close.assert_called_once_with()
        mockcon.commit.assert_called_once_with()
        mockcon.close.assert_called_once_with()
        self.assertEqual(res, ['1    koo', '3    yo', '4    val val'])


if __name__ == '__main__':
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_lookup
----------------------------------

Tests for `lookup` module.
"""
import os
import shutil
import tempfile
import sys
import unittest
import os
import configparser

from ncmirtools.config import ConfigMissingError
from ncmirtools.config import NcmirToolsConfig


class TestConfig(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_home_directory(self):
        con = NcmirToolsConfig()

        path = os.path.expanduser('~')
        self.assertEqual(con.get_home_directory(), path)

        con.set_home_directory('/foo')
        self.assertEqual(con.get_home_directory(), '/foo')

    def test_get_config_file(self):
        con = NcmirToolsConfig()

        path = os.path.expanduser('~')
        self.assertEqual(con.get_home_directory(), path)
        self.assertEqual(con.get_config_file(),
                         os.path.join(path,
                                      NcmirToolsConfig.CONFIG_FILE))

    def test_get_config_no_config_file_exists(self):
        temp_dir = tempfile.mkdtemp()
        try:
            con = NcmirToolsConfig()
            con.set_home_directory(temp_dir)

            try:
                config = con.get_config()
                self.fail('Expected ConfigMissingError')
            except ConfigMissingError as e:
                self.assertEqual(str(e),
                                 'No configuration file found here: ' +
                                 os.path.join(temp_dir,
                                              con.get_config_file()))
        finally:
            shutil.rmtree(temp_dir)

    def test_get_config_valid_config(self):
        temp_dir = tempfile.mkdtemp()
        try:
            con = NcmirToolsConfig()
            con.set_home_directory(temp_dir)

            initialcon = configparser.ConfigParser()
            initialcon.add_section(NcmirToolsConfig.POSTGRES_SECTION)
            initialcon.set(NcmirToolsConfig.POSTGRES_SECTION,
                           NcmirToolsConfig.POSTGRES_USER,
                           'bob')
            f = open(con.get_config_file(), 'w')
            initialcon.write(f)
            f.flush()
            f.close()
            con = NcmirToolsConfig()
            con.set_home_directory(temp_dir)
            config = con.get_config()

            val = config.get(NcmirToolsConfig.POSTGRES_SECTION,
                             NcmirToolsConfig.POSTGRES_USER)

            self.assertEqual(val, 'bob')
        finally:
            shutil.rmtree(temp_dir)

if __name__ == '__main__':
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_lookup
----------------------------------

Tests for `lookup` module.
"""
import os
import shutil
import tempfile
import sys
import re
import unittest

from ncmirtools.lookup import DirectoryForId
from ncmirtools.lookup import DirectorySearchPathError
from ncmirtools.lookup import InvalidMicroscopyProductIdError
from ncmirtools.lookup import InvalidProjectIdError



class TestLookup(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_DirectoryForId_constructor(self):

        # try passing none to constructor
        try:
            DirectoryForId(None)
            self.fail('Expected DirectorySearchPathError')
        except DirectorySearchPathError as e:
            self.assertEqual(str(e),
                             'search_path passed into constructor '
                             'cannot be None')

        # try passing invalid search_path
        bad_path = '/c/<VOLUME_ID>/<PROJECT_ID>/hi'
        try:

            DirectoryForId(bad_path)
            self.fail('Expected DirectorySearchPathError')
        except DirectorySearchPathError as e:
            self.assertEqual(str(e),
                             'Invalid search_path passed into '
                             'constructor : ' + bad_path)

        # try valid path
        DirectoryForId(
            DirectoryForId.PROJECT_DIR)

    def test__get_matching_directories(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # try Passing None as arguments
            dmp = DirectoryForId(
                DirectoryForId.PROJECT_DIR)
            self.assertEqual(len(dmp._get_matching_directories(None, None)),
                             0)
            self.assertEqual(len(dmp._get_matching_directories(temp_dir,
                                                               None)),
                             0)

            # try on empty dir with exact match on and off
            res = dmp._get_matching_directories(temp_dir, 'foo')
            self.assertEqual(len(res), 0)

            res = dmp._get_matching_directories(temp_dir, 'foo',
                                                exactmatch=True)
            self.assertEqual(len(res), 0)

            foodir = os.path.join(temp_dir, 'microscopy_12345')
            os.makedirs(foodir)

            # try with file as prefix
            afile = os.path.join(temp_dir, 'somefile')
            open(afile, 'a').close()
            res = dmp._get_matching_directories(temp_dir, 'somefile')
            self.assertEqual(len(res), 0)

            res = dmp._get_matching_directories(temp_dir, 'somefile',
                                                exactmatch=True)
            self.assertEqual(len(res), 0)

            # try with one non exact match
            res = dmp._get_matching_directories(temp_dir, 'microscopy_')
            self.assertEqual(len(res), 1)
            self.assertEqual(res[0], foodir)

            # try with exactmatch set to None
            res = dmp._get_matching_directories(temp_dir, 'microscopy_',
                                                exactmatch=None)
            self.assertEqual(len(res), 1)
            self.assertEqual(res[0], foodir)

            # try with exactmatch set to True but not an actual match
            res = dmp._get_matching_directories(temp_dir, 'microscopy_',
                                                exactmatch=True)
            self.assertEqual(len(res), 0)

            # try with exact match set to True and is an exact match
            res = dmp._get_matching_directories(temp_dir, 'microscopy_12345',
                                                exactmatch=True)
            self.assertEqual(len(res), 1)
            self.assertEqual(res[0], foodir)

            # try with exact match set to False and is an exact match
            res = dmp._get_matching_directories(temp_dir, 'microscopy_12345',
                                                exactmatch=False)
            self.assertEqual(len(res), 1)
            self.assertEqual(res[0], foodir)

            # try with multiple matching directories
            footwodir = os.path.join(temp_dir, 'microscopy_66')
            os.makedirs(footwodir)
            nonmatchdir = os.path.join(temp_dir, 'icroscopy_')
            os.makedirs(nonmatchdir)

            res = dmp._get_matching_directories(temp_dir, 'microscopy_',)
            self.assertEqual(len(res), 2)
            self.assertTrue(foodir in res)
            self.assertTrue(footwodir in res)

            res = dmp._get_matching_directories(temp_dir, 'microscopy_66',
                                                exactmatch=True)
            self.assertEqual(len(res), 1)
            self.assertTrue(footwodir in res)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_directory_for_microscopy_product_id(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # TODO: Make re.sub platform agnostic
            pdir = re.sub('^/', '',
                          DirectoryForId.PROJECT_DIR)

            sp = os.path.join(temp_dir, pdir)
            dmp = DirectoryForId(sp)

            # test mpid is None
            try:
                dmp.get_directory_for_microscopy_product_id(None)
                self.fail('Expected InvalidMicroscopyProductIdError')
            except InvalidMicroscopyProductIdError as e:
                self.assertEqual(str(e), 'microscopy product id cannot be '
                                         'None')

            # try on empty dir
            res = dmp.get_directory_for_microscopy_product_id(12345)
            self.assertEqual(len(res), 0)

            # TODO: Make path platform agnostic
            # try with 1 volume and no project
            volone = os.path.join(temp_dir, 'ccdbprod/ccdbprod1')
            os.makedirs(volone)
            res = dmp.get_directory_for_microscopy_product_id(12345)
            self.assertEqual(len(res), 0)

            # TODO: Make path platform agnostic
            # try with 1 volume, 1 project
            projone = os.path.join(volone, 'home/CCDB_DATA_USER.portal/'
                                           'CCDB_DATA_USER/acquisition/'
                                           'project_200000')
            os.makedirs(projone)
            res = dmp.get_directory_for_microscopy_product_id(12345)
            self.assertEqual(len(res), 0)

            # try with 1 volume 1 project, 1 microscopy id
            mpidone = os.path.join(projone, 'microscopy_12345')
            os.makedirs(mpidone)
            res = dmp.get_directory_for_microscopy_product_id(12345)
            self.assertEqual(len(res), 1)
            self.assertTrue(mpidone in res)

            # TODO: Make path platform agnostic
            # try with multiple volumes, projects & microscopy ids
            for val in range(2, 20, 1):
                avol = os.path.join(temp_dir, 'ccdbprod/ccdbprod' + str(val))
                os.makedirs(avol)
                for prj in range(300000, 300005, 1):
                    aprj = os.path.join(avol,
                                        'home/CCDB_DATA_USER.portal/'
                                        'CCDB_DATA_USER/acquisition/'
                                        'project_' + str(prj))
                    os.makedirs(aprj)
                    for mp in range(1, 5, 1):
                        ampid = os.path.join(aprj, 'microscopy_' + str(mp))
                        os.makedirs(ampid)
            res = dmp.get_directory_for_microscopy_product_id(12345)
            self.assertEqual(len(res), 1)
            self.assertTrue(mpidone in res)

            res = dmp.get_directory_for_microscopy_product_id(4)
            self.assertEqual(len(res), 90)

        finally:
            shutil.rmtree(temp_dir)

    def test_get_directory_for_project_id(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # TODO: Make re.sub platform agnostic
            pdir = re.sub('^/', '',
                          DirectoryForId.PROJECT_DIR)

            sp = os.path.join(temp_dir, pdir)
            dmp = DirectoryForId(sp)

            # test mpid is None
            try:
                dmp.get_directory_for_project_id(None)
                self.fail('Expected InvalidProjectIdError')
            except InvalidProjectIdError as e:
                self.assertEqual(str(e), 'project id cannot be '
                                         'None')

            # try on empty dir
            res = dmp.get_directory_for_project_id(12345)
            self.assertEqual(len(res), 0)

            # TODO: Make path platform agnostic
            # try with 1 volume and no project
            volone = os.path.join(temp_dir, 'ccdbprod/ccdbprod1')
            os.makedirs(volone)
            res = dmp.get_directory_for_project_id(12345)
            self.assertEqual(len(res), 0)

            # TODO: Make path platform agnostic
            # try with 1 volume, 1 project
            projone = os.path.join(volone, 'home/CCDB_DATA_USER.portal/'
                                           'CCDB_DATA_USER/acquisition/'
                                           'project_12345')
            os.makedirs(projone)
            res = dmp.get_directory_for_project_id(12345)
            self.assertEqual(len(res), 1)
            self.assertTrue(projone in res)

            # TODO: Make path platform agnostic
            # try with multiple volumes, projects
            for val in range(2, 20, 1):
                avol = os.path.join(temp_dir, 'ccdbprod/ccdbprod' + str(val))
                os.makedirs(avol)
                for prj in range(300000, 300005, 1):
                    aprj = os.path.join(avol,
                                        'home/CCDB_DATA_USER.portal/'
                                        'CCDB_DATA_USER/acquisition/'
                                        'project_' + str(prj))
                    os.makedirs(aprj)
                    for mp in range(1, 5, 1):
                        ampid = os.path.join(aprj, 'microscopy_' + str(mp))
                        os.makedirs(ampid)
            res = dmp.get_directory_for_project_id(12345)
            self.assertEqual(len(res), 1)
            self.assertTrue(projone in res)

            res = dmp.get_directory_for_project_id(300000)
            self.assertEqual(len(res), 18)


        finally:
            shutil.rmtree(temp_dir)

if __name__ == '__main__':
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_lookup
----------------------------------

Tests for `lookup` module.
"""
import re
import os
import sys
import logging
import tempfile
import shutil
import unittest

from ncmirtools import projectdir
from ncmirtools.lookup import DirectoryForId


class TestProjectdir(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_setup_logging(self):
        params = projectdir.Parameters()
        params.loglevel = 'DEBUG'
        projectdir._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.DEBUG)
        self.assertEqual(logging.getLogger
                         ('ncmirtools.projectdir').getEffectiveLevel(),
                         logging.DEBUG)
        self.assertEqual(logging.getLogger
                         ('ncmirtools.lookup').getEffectiveLevel(),
                         logging.DEBUG)

        params.loglevel = 'WARNING'
        projectdir._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.WARNING)
        self.assertEqual(logging.getLogger
                         ('ncmirtools.projectdir').getEffectiveLevel(),
                         logging.WARNING)
        self.assertEqual(logging.getLogger
                         ('ncmirtools.lookup').getEffectiveLevel(),
                         logging.WARNING)

        params.loglevel = 'INFO'
        projectdir._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.INFO)

        params.loglevel = 'ERROR'
        projectdir._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.ERROR)

        params.loglevel = 'CRITICAL'
        projectdir._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.CRITICAL)

    def test_parse_arguments(self):
        pargs = projectdir._parse_arguments('hello', ['12345'])
        self.assertEqual(pargs.projectid, '12345')
        self.assertEqual(pargs.loglevel, 'WARNING')
        self.assertTrue(pargs.prefixdir.startswith(os.sep + 'ccdbprod'))

        pargs = projectdir._parse_arguments('hello', ['1xx', '--log',
                                                  'DEBUG', '--prefixdir',
                                                  'hi'])
        self.assertEqual(pargs.projectid, '1xx')
        self.assertEqual(pargs.loglevel, 'DEBUG')
        self.assertEqual(pargs.prefixdir, 'hi')

    def test_run_lookup(self):

        # simple test where we raise an exception
        self.assertEqual(projectdir._run_lookup(None, None), 2)

        # test with non matching mpid
        self.assertEqual(projectdir._run_lookup(
            DirectoryForId.PROJECT_DIR, 'doesnotexistxx'), 1)

        # test with a match
        temp_dir = tempfile.mkdtemp()
        try:
            pdir = re.sub('^/', '',
                          DirectoryForId.PROJECT_DIR)
            mpdir = os.path.join(temp_dir, 'ccdbprod', 'ccdbprod1',
                                 'home', 'CCDB_DATA_USER.portal',
                                 'CCDB_DATA_USER', 'acquisition',
                                 'project_12345', 'microscopy_222')
            os.makedirs(mpdir)
            self.assertEqual(projectdir._run_lookup(os.path.join(temp_dir, pdir),
                                                '12345'), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_main(self):
        self.assertEqual(projectdir.main(), 1)


if __name__ == '__main__':
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_lookup
----------------------------------

Tests for `lookup` module.
"""
import re
import os
import sys
import logging
import tempfile
import shutil
import unittest

from ncmirtools import projectsearch


class TestProjectdir(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_setup_logging(self):
        params = projectsearch.Parameters()
        params.loglevel = 'DEBUG'
        projectsearch._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.DEBUG)
        self.assertEqual(logging.getLogger
                         ('ncmirtools.projectsearch').getEffectiveLevel(),
                         logging.DEBUG)
        self.assertEqual(logging.getLogger
                         ('ncmirtools.lookup').getEffectiveLevel(),
                         logging.DEBUG)

        params.loglevel = 'WARNING'
        projectsearch._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.WARNING)
        self.assertEqual(logging.getLogger
                         ('ncmirtools.projectsearch').getEffectiveLevel(),
                         logging.WARNING)
        self.assertEqual(logging.getLogger
                         ('ncmirtools.lookup').getEffectiveLevel(),
                         logging.WARNING)

        params.loglevel = 'INFO'
        projectsearch._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.INFO)

        params.loglevel = 'ERROR'
        projectsearch._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.ERROR)

        params.loglevel = 'CRITICAL'
        projectsearch._setup_logging(params)
        self.assertEqual(params.numericloglevel, logging.CRITICAL)

    def test_parse_arguments(self):
        pargs = projectsearch._parse_arguments('hello', ['12345'])
        self.assertEqual(pargs.keyword, '12345')
        self.assertEqual(pargs.loglevel, 'WARNING')
        self.assertEqual(pargs.homedir, '~')

        pargs = projectsearch._parse_arguments('hello', ['1xx', '--log',
                                                  'DEBUG', '--homedir',
                                                         'foo'])
        self.assertEqual(pargs.keyword, '1xx')
        self.assertEqual(pargs.loglevel, 'DEBUG')
        self.assertEqual(pargs.homedir, 'foo')

    def test_main(self):
        self.assertTrue(projectsearch.main() >= 0)


if __name__ == '__main__':
    sys.exit(unittest.main())



#! /usr/bin/env python

import os
import sys
import argparse
import ncmirtools
import logging

from ncmirtools.lookup import DirectoryForId

LOG_FORMAT = "%(asctime)-15s %(levelname)s %(name)s %(message)s"

# create logger
logger = logging.getLogger('ncmirtools.projectdir')

DIR_NOT_FOUND_MSG = 'Directory not found'


class Parameters(object):
    """Placeholder class for parameters
    """
    pass


def _setup_logging(theargs):
    """hi
    """
    theargs.logformat = LOG_FORMAT
    theargs.numericloglevel = logging.NOTSET
    if theargs.loglevel == 'DEBUG':
        theargs.numericloglevel = logging.DEBUG
    if theargs.loglevel == 'INFO':
        theargs.numericloglevel = logging.INFO
    if theargs.loglevel == 'WARNING':
        theargs.numericloglevel = logging.WARNING
    if theargs.loglevel == 'ERROR':
        theargs.numericloglevel = logging.ERROR
    if theargs.loglevel == 'CRITICAL':
        theargs.numericloglevel = logging.CRITICAL

    logger.setLevel(theargs.numericloglevel)
    logging.basicConfig(format=theargs.logformat)

    logging.getLogger('ncmirtools.projectdir').setLevel(theargs.numericloglevel)
    logging.getLogger('ncmirtools.lookup').setLevel(theargs.numericloglevel)


def _parse_arguments(desc, args):
    """Parses command line arguments using argparse.
    """
    parsed_arguments = Parameters()

    help_formatter = argparse.RawDescriptionHelpFormatter
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=help_formatter)
    parser.add_argument("projectid", help='Project Id to look for on file '
                                          'system')
    parser.add_argument("--log", dest="loglevel", choices=['DEBUG',
                        'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
                        help="Set the logging level (default WARNING)",
                        default='WARNING')

    parser.add_argument('--prefixdir',
                        default=DirectoryForId.PROJECT_DIR,
                        help='Defines the search path. Normally this does not '
                             'need to be adjusted. This script converts '
                             '<VOLUME_ID> and <MP_ID> into wildcards '
                             'which can be any value and <PROJECT_ID>  is the '
                             'value that will be matched to the projectid '
                             'value '
                             'set on the command line. (default ' +
                             DirectoryForId.PROJECT_DIR)

    parser.add_argument('--version', action='version',
                        version=('%(prog)s ' + ncmirtools.__version__))

    return parser.parse_args(args, namespace=parsed_arguments)


def _run_lookup(prefixdir, projectid):
    """Performs search for directory
    :param prefixdir: Directory search path
    :param mpid: microcsopy product id to use to find directory
    :returns: exit code for program
    """
    try:
        dmp = DirectoryForId(prefixdir)
        prj_dirs = dmp.get_directory_for_project_id(projectid)
        if len(prj_dirs) > 0:
            for entry in prj_dirs:
                sys.stdout.write(entry + os.linesep)
            return 0

        sys.stderr.write(DIR_NOT_FOUND_MSG + os.linesep)
        return 1
    except Exception:
        logger.exception("Error caught exception")
        return 2


def main():
    desc = """
              Version {version}

              Outputs the directory, or directories where the given
              <projectid> or Project id resides on the filesystem.

              If multiple directories match they will be output separated by
              newline character.

              If no path matching <projectid> is found this program will output
              to standard error the message '{dirnotfound}' and
              exit with value 1.

              If there is an error this program will output a message and
              exit with value 2.

              Example Usage:

              projectdir.py 2080

              """.format(version=ncmirtools.__version__,
                         dirnotfound=DIR_NOT_FOUND_MSG)

    theargs = _parse_arguments(desc, sys.argv[1:])
    theargs.program = sys.argv[0]
    theargs.version = ncmirtools.__version__
    _setup_logging(theargs)
    try:
        return _run_lookup(theargs.prefixdir, theargs.projectid)
    finally:
        logging.shutdown()


if __name__ == '__main__':
    sys.exit(main())



#! /usr/bin/env python

import os
import sys
import argparse
import ncmirtools
import logging

from ncmirtools.lookup import ProjectSearchViaDatabase
from ncmirtools.config import NcmirToolsConfig
from ncmirtools.config import ConfigMissingError

LOG_FORMAT = "%(asctime)-15s %(levelname)s %(name)s %(message)s"

# create logger
logger = logging.getLogger('ncmirtools.projectsearch')

NO_PROJECTS_FOUND_MSG = 'No matching projects found'

class Parameters(object):
    """Placeholder class for parameters
    """
    pass


def _setup_logging(theargs):
    """hi
    """
    theargs.logformat = LOG_FORMAT
    theargs.numericloglevel = logging.NOTSET
    if theargs.loglevel == 'DEBUG':
        theargs.numericloglevel = logging.DEBUG
    if theargs.loglevel == 'INFO':
        theargs.numericloglevel = logging.INFO
    if theargs.loglevel == 'WARNING':
        theargs.numericloglevel = logging.WARNING
    if theargs.loglevel == 'ERROR':
        theargs.numericloglevel = logging.ERROR
    if theargs.loglevel == 'CRITICAL':
        theargs.numericloglevel = logging.CRITICAL

    logger.setLevel(theargs.numericloglevel)
    logging.basicConfig(format=theargs.logformat)

    logging.getLogger('ncmirtools.config').setLevel(theargs.numericloglevel)
    logging.getLogger('ncmirtools.projectsearch').setLevel(theargs.numericloglevel)
    logging.getLogger('ncmirtools.lookup').setLevel(theargs.numericloglevel)


def _parse_arguments(desc, args):
    """Parses command line arguments using argparse.
    """
    parsed_arguments = Parameters()

    help_formatter = argparse.RawDescriptionHelpFormatter
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=help_formatter)
    parser.add_argument("keyword", help='keyword to search for in name or '
                                     ' description of project')
    parser.add_argument("--log", dest="loglevel", choices=['DEBUG',
                        'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
                        help="Set the logging level (default WARNING)",
                        default='WARNING')
    parser.add_argument("--homedir", help='Sets alternate home directory'
                                          'under which the ' +
                                          NcmirToolsConfig.CONFIG_FILE +
                                          ' is loaded (default ~)',
                        default='~')
    parser.add_argument('--version', action='version',
                        version=('%(prog)s ' + ncmirtools.__version__))

    return parser.parse_args(args, namespace=parsed_arguments)


def _run_search_database(keyword, homedir):
    """Performs search for directory
    :param prefixdir: Directory search path
    :param mpid: microcsopy product id to use to find directory
    :returns: exit code for program
    """
    try:
        config = NcmirToolsConfig()
        config.set_home_directory(os.path.expanduser(homedir))

        search = ProjectSearchViaDatabase(config.get_config())
        res = search.get_matching_projects(keyword)
        if len(res) > 0:
            for entry in res:
                sys.stdout.write(entry + os.linesep)
            return 0

        sys.stderr.write(NO_PROJECTS_FOUND_MSG + os.linesep)
        return 1
    except ConfigMissingError:
        sys.stderr.write('\nERROR: Configuration file missing.\n'
                         ' Please run projectsearch.py --help for '
                         'information on how\n to create a configuration '
                         'file\n\n')
        return 3
    except Exception:
        logger.exception("Error caught exception")
        return 2


def main():
    config = NcmirToolsConfig()
    desc = """
              Version {version}

              Given a <keyword> this script searches the database for any
              projects where the keyword exists in the project name or
              description. The search is case insensitive. The matching
              projects will be output separated by new line characters
              as follows:

              <project id>   <project name>

              If no project matches the <keyword> is found this program will
              output to standard error the message '{projectnotfound}'
              and exit with value 1.

              If there is an unknown error this program will output a message
              and exit with value 2.

              If {config_file}
              is missing then this program will output a message and exit
              with value 3.

              Example Usage:

              projectsearch.py 'yo'
              2033 yo project
              2047 you better believe it

              NOTE:

              This script requires a configuration file which contains
              the information to connect to the database.

              For this account the file should be located here:
              {config_file}

              and should have the following format:

              [{db}]
              {user} = <database username>
              {password} = <database password>
              {port} = <database port ie 5432>
              {host} = <database host>
              {database} = <database name>

              """.format(version=ncmirtools.__version__,
                         projectnotfound=NO_PROJECTS_FOUND_MSG,
                         db=NcmirToolsConfig.POSTGRES_SECTION,
                         user=NcmirToolsConfig.POSTGRES_USER,
                         password=NcmirToolsConfig.POSTGRES_PASS,
                         port=NcmirToolsConfig.POSTGRES_PORT,
                         host=NcmirToolsConfig.POSTGRES_HOST,
                         database=NcmirToolsConfig.POSTGRES_DB,
                         config_file=config.get_config_file())

    theargs = _parse_arguments(desc, sys.argv[1:])
    theargs.program = sys.argv[0]
    theargs.version = ncmirtools.__version__
    _setup_logging(theargs)
    try:
        return _run_search_database(theargs.keyword, theargs.homedir)
    finally:
        logging.shutdown()


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "argparse",
    "configparser",
    "pg8000"
]

test_requirements = [
    "argparse",
    "pg8000",
    "configparser",
    "mock"
]

setup(
    name='ncmirtools',
    version='0.2.0',
    description="Set of commandline tools for NCMIR",
    long_description=readme + '\n\n' + history,
    author="Christopher Churas",
    author_email='churas.camera@gmail.com',
    url='https://github.com/CRBS/ncmirtools',
    packages=[
        'ncmirtools',
    ],
    package_dir={'ncmirtools':
                 'ncmirtools'},
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    keywords='ncmirtools',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    scripts = [ 'ncmirtools/mpidir.py',
                'ncmirtools/projectsearch.py',
                'ncmirtools/projectdir.py'
              ],
    test_suite='tests',
    tests_require=test_requirements
)



__author__ = 'churas'


import os
import configparser
import logging


logger = logging.getLogger(__name__)


class ConfigMissingError(Exception):
    """Raised if configuration file is missing
    """
    pass


class NcmirToolsConfig(object):
    """Class provides access to ncmirtools configuration
    file which is stored in the user's home directory
    """
    CONFIG_FILE = '.ncmirtools.conf'

    POSTGRES_SECTION = 'postgres'
    POSTGRES_USER = 'user'
    POSTGRES_PASS = 'password'
    POSTGRES_HOST = 'host'
    POSTGRES_PORT = 'port'
    POSTGRES_DB = 'database'

    def __init__(self):
        """Constructor
        """
        self._homedir = os.path.expanduser('~')

    def set_home_directory(self, path):
        """Sets alternate home directory
        :param path: Alternate home directory path
        """
        self._homedir = path

    def get_home_directory(self):
        """Returns home directory path
        :returns: Path to home directory
        """
        return self._homedir

    def get_config_file(self):
        """Gets full path to configuration file
        :returns: Full path to configuration file under the home directory
                  of the user
        """
        return os.path.join(self._homedir,
                            NcmirToolsConfig.CONFIG_FILE)

    def get_config(self):
        """Gets configparser object loaded with configuration
        :returns output of configparser.ConfigParser() after it has been
                 loaded with configuration from `get_config_file()`
        :raises ConfigMissingError: If no configuration file is found
        """
        if not os.path.isfile(self.get_config_file()):
            raise ConfigMissingError('No configuration file found here: ' +
                                     self.get_config_file())

        parser = configparser.ConfigParser()
        parser.read(self.get_config_file())
        return parser



# -*- coding: utf-8 -*-


import os
import logging
import re
import pg8000

from ncmirtools.config import NcmirToolsConfig

logger = logging.getLogger(__name__)


class DirectorySearchPathError(Exception):
    """Raised when there is an error parsing Directory Search path
    """
    pass


class InvalidMicroscopyProductIdError(Exception):
    """Raised when MicroscopyProductId pass in is invalid
    """
    pass


class InvalidProjectIdError(Exception):
    """Raised when Project Id passed in is invalid
    """
    pass


class DirectoryForId(object):
    """Given a microscopy product id or project id
       instances of this class
       find corresponding directory on the filesystem
    """
    VOLUME_ID = '<VOLUME_ID>'
    MP_ID = '<MP_ID>'
    PROJECT_ID = '<PROJECT_ID>'

    # TODO: Make path platform agnostic ie use os.path.join
    PROJECT_DIR = ('/ccdbprod/ccdbprod<VOLUME_ID>/home'
                   '/CCDB_DATA_USER.portal/CCDB_DATA_USER'
                   '/acquisition/project_<PROJECT_ID>'
                   '/microscopy_<MP_ID>')

    def __init__(self, search_path):
        """Constructor
        :param search_path: string define search path. This path should have
                            values of `VOLUME_ID`, `PROJECT_ID`, and `MP_ID`
                             in it with the values in that order.
         See `PROJECT_DIR` value for default value
        :raises DirectorySearchPathError: If search_path parameter is None or
         invalid
        """
        self._search_path = search_path
        self._volpath = None
        self._projpath = None
        self._mpidpath = None

        if self._search_path is None:
            raise DirectorySearchPathError('search_path passed into '
                                           'constructor cannot be None')

        splitpath = re.split('^(.*)' +
                             DirectoryForId.VOLUME_ID +
                             '(.*)' +
                             DirectoryForId.PROJECT_ID +
                             '(.*)' +
                             DirectoryForId.MP_ID,
                             self._search_path)

        if len(splitpath) < 4:
            raise DirectorySearchPathError('Invalid search_path passed into '
                                           'constructor : ' + search_path)
        logger.debug(splitpath)
        self._volpath = splitpath[1]
        # TODO: Make path regex platform agnostic
        self._projpath = re.sub('^/', '', splitpath[2])
        self._mpidpath = re.sub('^/', '', splitpath[3])
        logger.debug('volpath=' + self._volpath + ' projpath=' +
                     self._projpath + 'mpidpath=' + str(self._mpidpath))

    def get_directory_for_microscopy_product_id(self, mpid):
        """Gets directory for microscopy product id `mpid`
        :param mpid: microscopy product id ie 5269524
        :returns: List of directories that match `mpid`
        """
        if mpid is None:
            raise InvalidMicroscopyProductIdError('microscopy product id '
                                                  'cannot be None')

        basedir = os.path.dirname(self._volpath)
        dirprefix = os.path.basename(self._volpath)
        match_vol_dirs = self._get_matching_directories(basedir,
                                                        dirprefix)
        logger.debug('vol dir count ' + str(len(match_vol_dirs)))
        project_dir_count = 0
        mp_dir_count = 0
        final_matches = []
        for vol_dir in match_vol_dirs:
            raw_prj_dir = os.path.join(vol_dir, self._projpath)
            basedir = os.path.dirname(raw_prj_dir)
            dirprefix = os.path.basename(raw_prj_dir)
            match_prj_dirs = self._get_matching_directories(basedir,
                                                            dirprefix)
            project_dir_count += len(match_prj_dirs)

            for mp_dir in match_prj_dirs:
                raw_mp_dir = os.path.join(mp_dir, self._mpidpath)
                basedir = os.path.dirname(raw_mp_dir)
                dirprefix = os.path.basename(raw_mp_dir) + str(mpid)
                match_mp_dirs = self._get_matching_directories(basedir,
                                                               dirprefix,
                                                               exactmatch=True)
                mp_dir_count += len(match_mp_dirs)
                if len(match_mp_dirs) > 0:
                    final_matches.extend(match_mp_dirs)

        logger.debug('project dir count ' + str(project_dir_count))
        logger.debug('mp_dir count ' + str(mp_dir_count))

        return final_matches

    def get_directory_for_project_id(self, projectid):
        """Gets directory for projectid id `projectid`
        :param projectid: microscopy product id ie 2080
        :returns: List of directories that match `projectid`
        """
        if projectid is None:
            raise InvalidProjectIdError('project id cannot be None')

        basedir = os.path.dirname(self._volpath)
        dirprefix = os.path.basename(self._volpath)
        match_vol_dirs = self._get_matching_directories(basedir,
                                                        dirprefix)
        logger.debug('vol dir count ' + str(len(match_vol_dirs)))
        project_dir_count = 0
        mp_dir_count = 0
        final_matches = []
        for vol_dir in match_vol_dirs:
            raw_prj_dir = os.path.join(vol_dir, self._projpath)
            basedir = os.path.dirname(raw_prj_dir)
            dirprefix = os.path.basename(raw_prj_dir + str(projectid))
            match_prj_dirs = self._get_matching_directories(basedir,
                                                            dirprefix,
                                                            exactmatch=True)
            project_dir_count += len(match_prj_dirs)
            if len(match_prj_dirs) > 0:
                final_matches.extend(match_prj_dirs)

        logger.debug('project dir count ' + str(project_dir_count))

        return final_matches

    def _get_matching_directories(self, basedir, prefix,
                                  exactmatch=False):
        """Gets list of directories under `basedir` matching `prefix`
        :param basedir: full path to directory to look under (ie
                        /ccdbprod)
        :param prefix: Any directory immediately under `basedir` that
                       starts with content of `prefix` is considered a

        :param exactmatch: Normally a match is found if directory under
                           `basedir` starts withe `prefix`, but if this
                           is True then only directories that exactly match
                           `prefix` are added to list
        :returns: List of strings that are full paths to directories matching
                  prefix
        """
        matching_dirs = []
        if basedir is None:
            logger.warning('basedir is None')
            return matching_dirs

        if prefix is None:
            logger.warning('prefix dir is None')
            return matching_dirs

        if exactmatch is None:
            logger.warning('exactmatch was set to None, using False')
            exactmatch = False

        logger.debug("thedir=" + basedir + " theprefix=" + prefix +
                     " exactmatch=" + str(exactmatch))

        if not os.path.isdir(basedir):
            logger.warning(basedir + ' is not a directory')
            return matching_dirs

        try:
            for entry in os.listdir(basedir):
                if exactmatch is False:
                    if entry.startswith(prefix):
                        fpath = os.path.join(basedir, entry)
                        if os.path.isdir(fpath):
                            matching_dirs.append(fpath)
                else:
                    if entry == prefix:
                        fpath = os.path.join(basedir, entry)
                        if os.path.isdir(fpath):
                            matching_dirs.append(fpath)
        except OSError:
            logger.exception('Caught Exception')
        return matching_dirs


class ProjectSearchViaDatabase(object):
    """Searches for Projects via Database
    """

    def __init__(self, config):
        """Constructor
        :param config: ConfigParser object with information to connect to
                       database.
        """
        self._config = config
        self._alt_conn = None

    def set_config(self, config):
        """Sets alternate config
        :param config: ConfigParser object with information to connect to
                       database.
        """
        self._config = config

    def set_alternate_connection(self, conn):
        """Sets alternate database connection
        :param conn: Alternate database connection
        """
        self._alt_conn = conn

    def _get_connection(self):
        """Gets connection to database
        :returns: Connection to database as Connection object
        """
        if self._alt_conn is not None:
            logger.info("Using alternate database connection")
            return self._alt_conn

        userval = self._config.get(NcmirToolsConfig.POSTGRES_SECTION,
                                NcmirToolsConfig.POSTGRES_USER)

        passval = self._config.get(NcmirToolsConfig.POSTGRES_SECTION,
                                   NcmirToolsConfig.POSTGRES_PASS)

        hostval = self._config.get(NcmirToolsConfig.POSTGRES_SECTION,
                                             NcmirToolsConfig.POSTGRES_HOST)

        portval = self._config.get(NcmirToolsConfig.POSTGRES_SECTION,
                                NcmirToolsConfig.POSTGRES_PORT)

        dbval = self._config.get(NcmirToolsConfig.POSTGRES_SECTION,
                                NcmirToolsConfig.POSTGRES_DB)

        conn = pg8000.connect(host=hostval, user=userval,
                              password=passval,
                              port=int(portval),
                              database=dbval)
        return conn

    def get_matching_projects(self, keyword):
        """Finds projects matching keyword
        :param keyword: Keyword to use to search for projects
        :returns: list of strings containing project id followed by project
                  name.  Ex: 20333    some project
        """
        conn = self._get_connection()
        cursor = conn.cursor()
        res = []
        try:
            if keyword is None:
                cursor.execute("SELECT Project_id,project_name FROM Project")
            else:
                cursor.execute("SELECT Project_id,project_name FROM Project "
                               "WHERE project_name ILIKE '%%" + keyword + "%%' OR "
                               " project_desc ILIKE '%%" + keyword + "%%'")
            for tuple in cursor.fetchall():
                res.append(str(tuple[0]) + '    ' + str(tuple[1]))
        finally:
            cursor.close()
            conn.commit()
            conn.close()

        return res